@@ -1,7 +1,10 @@
+from itertools import groupby
+
 from prices import PriceRange
 
-__all__ = ['InsufficientStockException', 'Item', 'ItemLine', 'ItemRange',
-           'ItemSet', 'StockedItem']
+__all__ = ['ClassifyingPartitioner', 'InsufficientStockException', 'Item',
+           'ItemLine', 'ItemRange', 'ItemSet', 'Partitioner', 'StockedItem',
+           'partition']
 
 
 class InsufficientStock(Exception):
@@ -17,7 +20,7 @@ class ItemRange(object):
     Represents a group of products like a product with multiple variants
     """
     def __iter__(self):
-        raise NotImplementedError()  # pragma: no cover
+        raise NotImplementedError()
 
     def get_price_per_item(self, item, **kwargs):
         return item.get_price(**kwargs)
@@ -35,7 +38,7 @@ class ItemSet(object):
     Represents a set of products like an order or a basket
     """
     def __iter__(self):
-        raise NotImplementedError()  # pragma: no cover
+        raise NotImplementedError()
 
     def get_subtotal(self, item, **kwargs):
         return item.get_total(**kwargs)
@@ -58,7 +61,7 @@ class ItemLine(object):
     Represents a single line of an order or basket
     """
     def get_price_per_item(self, **kwargs):
-        return NotImplemented  # pragma: no cover
+        raise NotImplementedError()
 
     def get_quantity(self, **kwargs):
         return 1
@@ -76,7 +79,7 @@ class Item(object):
     shirt)
     """
     def get_price_per_item(self, **kwargs):
-        return NotImplemented  # pragma: no cover
+        raise NotImplementedError()
 
     def get_price(self, **kwargs):
         return self.get_price_per_item(**kwargs)
@@ -98,6 +101,41 @@ class Partitioner(ItemSet):
     def __nonzero__(self):
         return bool(self.subject)
 
+    def __repr__(self):
+        return '%s(%r)' % (type(self).__name__, self.subject)
+
+
+class ClassifyingPartitioner(Partitioner):
+
+    def __iter__(self):
+        subject = sorted(self.subject, key=self.classify)
+        for classifier, items in groupby(subject, key=self.classify):
+            yield self.get_partition(classifier, items)
+
+    def classify(self, item):
+        raise NotImplementedError()
+
+    def get_partition(self, classifier, items):
+        return ItemList(items)
+
+
+class GroupingPartitioner(ClassifyingPartitioner):
+
+    def __init__(self, subject, keyfunc, partition_class):
+        self.keyfunc = keyfunc
+        self.partition_class = partition_class
+        super(GroupingPartitioner, self).__init__(subject)
+
+    def classify(self, item):
+        return self.keyfunc(item)
+
+    def get_partition(self, classifier, items):
+        return self.partition_class(items)
+
+
+def partition(subject, keyfunc, partition_class=ItemList):
+    return GroupingPartitioner(subject, keyfunc, partition_class)
+
 
 class StockedItem(Item):
 
